@@ -15,6 +15,7 @@ from sqlalchemy import (
     String,
     Time,
     Text,
+    UniqueConstraint
 )
 from sqlalchemy.sql.expression import desc
 from sqlalchemy import create_engine
@@ -33,12 +34,16 @@ class Storage(object):
 
         self._engine = new_engine()
         self._maker = sessionmaker(bind=self._engine)
-        self.session = self._maker()
+        self.session = self._new_session()
+
+    def _new_session(self):
+        return self._maker()
 
     def bootstrap(self):
         models = [
             Job,
             Event,
+            Category,
         ]
         for model in models:
             try:
@@ -52,15 +57,23 @@ class Storage(object):
         self.session.close()
         self._engine.dispose()
 
-    def add_job(self, name, interval_seconds, command, next_run=None, id=None):
+    def add_job(self, name, interval_seconds, command, next_run, id=None, category_id=None):
         job = Job()
         job.id = id
         job.name = name
+        job.next_run = next_run
         job.interval = datetime.timedelta(seconds=interval_seconds)
         job.command = command
+        job.category_id = category_id
         self.session.merge(job)
         self.session.commit()
 
+    def remove_job(self, job_id):
+        job = self.session.query(Job).filter_by(id=job_id).first()
+        if job:
+            self.session.delete(job)
+            self.session.commit()
+
     def add_event(self, job_id, datetime, run_id, type, host, return_code):
         event = Event()
         event.job_id = job_id
@@ -85,6 +98,9 @@ class Storage(object):
 
     def get_job(self, id):
         job = self.session.query(Job).filter_by(id=id).first()
+        return self._job_doc_for_model(job)
+
+    def _job_doc_for_model(self, job):
         return {
             'id': job.id,
             'name': job.name,
@@ -93,6 +109,24 @@ class Storage(object):
             'command': job.command,
         }
 
+
+    def jobs_for_category(self, id=None, name=None):
+        if id and name:
+            assert "Don't pass both id and name"
+        if name:
+            id = self.category_id_for_name(name)
+        for job in self.session.query(Job).filter_by(category_id=id):
+            yield self._job_doc_for_model(job)
+
+    def category_id_for_name(self, name):
+        category = self.session.query(Category).filter_by(name=name).first()
+        if category is None:
+            category = Category(name=name)
+            self.session.add(category)
+            self.session.commit()
+        return category.id
+
+
     def last_events_for_job(self, job_id, number):
         events = self.session.query(Event).filter_by(job_id=job_id).order_by(desc(Event.id)).limit(number)
         return event_models_to_docs(events)
@@ -120,7 +154,7 @@ class Storage(object):
             pass
 
     def get_unpublished_task(self):
-        session = self.session
+        session = self._new_session()
         to_run = or_(
             Job.next_run < datetime.datetime.utcnow(),
             Job.run_now == True,
@@ -128,27 +162,29 @@ class Storage(object):
 
         job = session.query(Job).filter(to_run).first()
         if job is None:
+            session.close()
             return
         print 'Found a job:', job.name, job.next_run
         to_run_at = job.next_run
+        while job.next_run < datetime.datetime.utcnow():
+            print 'Adding time!'
+            job.next_run += job.interval
+        print job.next_run
+        job_doc = {
+            'name': job.name,
+            'command': unicode(job.command),
+            'id': job.id,
+        }
+        job.run_now = False
+        me = '{0}.{1}'.format(socket.gethostname(), os.getpid())
+        job.locked_by = me
         try:
-            while job.next_run < datetime.datetime.utcnow():
-                print 'Adding time!'
-                job.next_run += job.interval
-            print job.next_run
-            job_doc = {
-                'name': job.name,
-                'command': unicode(job.command),
-                'id': job.id,
-            }
-            job.run_now = False
-            me = '{0}.{1}'.format(socket.gethostname(), os.getpid())
-            job.locked_by = me
             session.commit()
             self.publisher.publish(job.routing_key, job_doc, uuid4().hex)
         except Exception as exc:
             session.rollback()
             raise
+        session.close()
         return True
 
 def event_models_to_docs(events):
@@ -170,7 +206,9 @@ def new_engine():
 
 
 class Job(Base):
+
     __tablename__ = 'jobs'
+    __table_args__ = (UniqueConstraint('category_id', 'name'),{'mysql_engine':'InnoDB'})
 
     id = Column(Integer, primary_key=True)
     name = Column(CHAR(255))
@@ -180,11 +218,13 @@ class Job(Base):
     command = Column(Text())
     run_now = Column(Integer(1))
     locked_by = Column(CHAR(64))
+    category_id = Column(Integer)
 
 
 class Event(Base):
 
     __tablename__ = 'events'
+    __table_args__ = {'mysql_engine':'InnoDB'}
 
     id = Column(Integer, primary_key=True)
     job_id = Column(Integer)
@@ -193,3 +233,11 @@ class Event(Base):
     type = Column(CHAR(32))
     host= Column(CHAR(255))
     return_code = Column(Integer)
+
+class Category(Base):
+
+    __tablename__ = 'categories'
+    __table_args__ = {'mysql_engine':'InnoDB'}
+
+    id = Column(Integer, primary_key=True)
+    name = Column(CHAR(255), unique=True)
